

# screen_cyhd_snp
# ------------------------------------------------------------------------------
# 05/08/2013

screen_ax_snp <- function( verbose, ... ) {
    if( verbose ) {
        message( paste0( "[ ax2snpmatrix ]: ", ... ) );
    }
}


# ax2snpmatrix
# ------------------------------------------------------------------------------
# Function to create an object which contains a SnpMatrix and a mapping dataset
# thought a cohort of CEL files from Affymetrix Axiom

ax2snpmatrix <- function( calls.file, annotation.file, 
    output.type = "snpmatrix", output.name = NA, verbose=FALSE ) {

    output.type <- value( output.type, c( "snpmatrix", "plink" ) )

    # Checking input arguments
    if( output.type == "" ) {
        screen_gw_snp( TRUE, "Argument 'output.type' must be 'snpmatrix' or 'plink'." )
        stop( "HALT." )
    }
    if( output.type == "plink" & is.na( output.name ) ) {
        screen_gw_snp( TRUE, "If 'output.type' is 'plink', 'output.name' must be filled." )
        stop( "HALT." )   
    }
    # /Checking input arguments

    screen_ax_snp( verbose, "Loading calls file." );
    calls <- tryCatch( { read.table( calls.file, row.names=1, comment.char="#", header=TRUE, as.is=TRUE ); },
        warnning=function( w ) { screen_ax_snp( TRUE, "Invalid file provided in \"callsfile\"\n.", w ); stop( "HALT." ); },
        error=function( e ) { screen_ax_snp( TRUE, "Invalid file provided in \"callsfile\".\n", e ); stop( "HALT." ); } );

    screen_ax_snp( verbose, "Loading annotation file." );
    annot <- tryCatch( { read.csv( annotation.file, row.names=1, comment.char="#", header=TRUE ); },
        warnning=function( w ) { screen_ax_snp( TRUE, "Invalid file provided in \"annotation\".\n", w ); stop( "HALT." ); },
        error=function( e ) { screen_ax_snp( TRUE, "Invalid file provided in \"annotation\".\n", e ); stop( "HALT." ); } );

    

    if( output.type == "snpmatrix" ) {
        
        screen_ax_snp( verbose, "Coercing calls to SnpMatrix." );
        calls.ind <- new( "SnpMatrix", t( as( calls, "matrix" ) ) );
        
        screen_ax_snp( verbose, "Creating container." );
        object <- list();
        object$map <- data.frame( annot$Chromosome, annot$dbSNP.RS.ID, as.numeric( annot$Physical.Position ), annot$Allele.A, annot$Allele.B )
        colnames( object$map ) <- c( "chromosome", "dbSNP", "position", "allele.1", "allele.2" );
        rownames( object$map ) <- rownames( annot );
        object$genotype <- calls.ind;
        object$map <- object$map[ colnames( object$genotype ), ]
        
        return( object );
    }
    else {
        
        allA <- as.character( annot$Allele.A );
        allComplA <- c( "A","C","G","T" )[ match( allA, c( "T", "G", "C", "A" ) ) ];
        forwA <- ifelse( annot$Strand=="+", allA, allComplA );
        
        allB <- as.character( annot$Allele.B );
        allComplB <- c( "A","C","G","T" )[ match( allB, c( "T", "G", "C", "A" ) ) ]
        forwB <- ifelse( annot$Strand=="+", allB, allComplB )
        
        
        
        alleleA <- alleleB <- matrix( "", nrow=nrow( calls ), ncol=ncol( calls ), dimnames=dimnames( calls ) );
        for( ii in 1:ncol( calls ) ) {
            alleleA[ , ii ] <- ifelse( calls[ , ii ] < 3, forwA, forwB );
            alleleB[ , ii ] <- ifelse( calls[ , ii ] < 2, forwA, forwB );
        }
        
        
        
        p1 <- seq( 1, ncol( calls )*2, 2 );
        p2 <- seq( 2, ncol( calls )*2, 2 );
        
        tped <- matrix( "", ncol=ncol( calls ) * 2, nrow=nrow( calls ) );
        for( ii in 1:ncol( calls ) ) {
            tped[ , p1[ ii ] ] <- alleleA[ , ii ];
            tped[ , p2[ ii ] ] <- alleleB[ , ii ];
        }
        
        rownames( tped ) <- rownames( calls );
        
        
        
        screen_ax_snp( verbose, "Selecting SNP data from annotation file." );
        annot <- annot[ rownames( calls ), ];
        annot <- data.frame( annot$Chromosome, annot$dbSNP.RS.ID, as.numeric( annot$ChrX.pseudo.autosomal.region.1 ),  as.numeric( annot$Physical.Position ) );
        
        
        screen_ax_snp( verbose, "Flatting SNP data with TPED content." );
        tped <- cbind( annot, tped );
        
        
        screen_ax_snp( verbose, paste0( "Writing TPED file to disk '", output.name, ".tped'." ) );
        write.table( tped, file=paste0( output.name, ".tped" ), quote=FALSE, row.names=FALSE, col.names=FALSE );
    }

    return( NA );
}