
# screen_cyhd_snp
# ---------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# 17/07/2013

screen_cyhd_snp <- function( verbose, ... ) {
    if( verbose ) {
        message( paste0( "[ cy2snpmatrix ]: ", ... ) );
    }
}


# cy2snpmatrix
# ---------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# 12/07/2013
# Function to do the hight level analysis perform on cyhd.cychp.txt files generated from cy2apt. It creates an object which contains a
# SnpMatrix and a mapping dataset thought an original cohort of CEL files from Affymetrix CytoSCanHD.

cy2snpmatrix <- function( cychp.files, annotation.file, cychp.attime = 4,
    output.type = "snpmatrix", output.name = NA, verbose = FALSE ) {

	if(!CheckPython()) {
		screen_cyhd_snp(TRUE, "This function required python and python-pandas. Please install them before run this function.")
		stop( "HALT." )
	}
  
	SetCores(cychp.attime)
	
    if( cychp.files == "" ) {
        cychp.files = "./";
    }

    output.type <- value( output.type, c( "snpmatrix", "plink" ) )
	output.temp <- RandomName(prefix="tmp.cy.snpmatrix.")

    files.complete <- list.files( cychp.files, pattern=".cyhd.cychp.txt", full.names=TRUE );

    # Checking input arguments
    if( length( files.complete ) == 0 ) {
        screen_cyhd_snp( TRUE, "No 'cychp' files in folder '", cychp.files, "'." )
        stop( "HALT." );
    }
    if( output.type == "" ) {
        screen_cyhd_snp( TRUE, "Argument 'output.type' must be 'snpmatrix' or 'plink'." )
        stop( "HALT." )
    }
    if( output.type == "plink" & is.na( output.name ) ) {
        screen_cyhd_snp( TRUE, "If 'output.type' is 'plink', 'output.name' must be filled." )
        stop( "HALT." )
    }
    # /Checking input arguments


    # Showing the input arguments
    m <- paste0( "\n-------------------------\n",
        " - Input:        ", cychp.files, "\n",
        " - Annotation:   ", annotation.file, "\n",
        " - Temp path:    ", output.temp, "\n",
        " - cychp x time: ", cychp.attime, "\n",
        " - Output:       ", output.type );
    if( output.type == "plink" ) {
        m <- paste0( m, "\n - Output f.:    ", output.name )
    }
    m <- paste0( m, "\n-------------------------" );
    screen_cyhd_snp( verbose, m );
    # /Showing the input arguments


    cychp.files <- list.files( cychp.files, pattern=".cyhd.cychp.txt", full.names=TRUE );


    screen_cyhd_snp( verbose, "Loading annotation file." );
    annot <- tryCatch( { read.csv( annotation.file, row.names=1, comment.char="#", header=TRUE ); },
        warnning=function( w ) { screen_cyhd_snp( TRUE, "Invalid file provided in \"annotation.file\".\n" ); stop( "HALT." ); },
        error=function( e ) { screen_cyhd_snp( TRUE, "Invalid file provided in \"annotation.file\".\n" ); stop( "HALT." ); } )

    dir.create( output.temp, showWarnings=FALSE );


    if( output.type == "snpmatrix" ) {
        # if no output.type is specified we create an object containing the
        # map content and the SnpMatrix to return it.

        screen_cyhd_snp( verbose, "'output.type' set as 'snpmatrix', a container for ",
            "MAP and SnpMatrix will be created and returned." );

        map <- data.frame( annot$Chromosome, annot$dbSNP.RS.ID, as.numeric( annot$Physical.Position ), annot$Allele.A, annot$Allele.B )
        colnames( map ) <- c( "chromosome", "dbSNP", "position", "allele.1", "allele.2" );
        rownames( map ) <- rownames( annot );

        screen_cyhd_snp( verbose, "Starting analysis." );
        calls <- plapply( 1:length( cychp.files ), extract_genotypes,
            cychp.files, output.temp, verbose)#, mc.cores=cychp.attime );

        common <- Reduce( intersect, Map( function( x ) { x$names; }, calls ) )
        calls <- Map( function( x ){ x$genotype[ common ] }, calls )


        screen_cyhd_snp( verbose, "Flatting data (*)." );

        calls.ind <- data.frame( calls[[ 1 ]] );

        for( ii in 2:length( calls ) ) {
            calls.ind <- cbind( calls.ind, calls[[ ii ]] );
        }

        colnames( calls.ind ) <- gsub( ".cyhd.cychp.txt", "", basename( cychp.files ) );
        rownames( calls.ind ) <- common #calls[[ ii ]]$names;


        screen_cyhd_snp( verbose, "Coercing to a SnpMatrix." );
        calls.sm <- new( "SnpMatrix", t( as( calls.ind, "matrix" ) ) );

        screen_cyhd_snp( verbose, "Creating container." );
        object <- list();
        object$map <- map;
        object$genotype <- calls.sm;
        object$map <- object$map[ colnames( object$genotype ), ];

        return( object );

    }
    else {
        # if output.type is specified the output.name will contain the name
        # use to create the tped file (plink output).

        screen_cyhd_snp( verbose, "'output.type' set as 'plink', a TPED file ",
            "will be created and stored to disk." );


        screen_cyhd_snp( verbose, "Creating the TPED content," );
        calls.ind <-  process_files_as_ped( cychp.files, cychp.attime,
            as.character( annot$Allele.A ), as.character( annot$Allele.B ),
            as.character( annot$Strand ), output.temp, verbose );


        screen_cyhd_snp( verbose, "Selecting SNP data from annotation file." );
        annot <- annot[ rownames( calls.ind ), ];
        tped <- data.frame( annot$Chromosome, annot$dbSNP.RS.ID, as.numeric( annot$ChrX.pseudo.autosomal.region.1 ),  as.numeric( annot$Physical.Position ) );


        screen_cyhd_snp( verbose, "Flatting SNP data with TPED content." );
        tped <- cbind( tped, calls.ind );


        screen_cyhd_snp( verbose, paste0( "Writing TPED file to disk '", output.type, ".tped'." ) );
        write.table( tped, file=paste0( output.name, ".tped" ), quote=FALSE, row.names=FALSE, col.names=FALSE );
    }

    unlink( output.temp, recursive=TRUE, force=TRUE );

    return( NA )
}

process_files_as_matrix <- function( cychp.files, cychp.names, cychp.attime, output.temp, verbose ) {

    genotypes <- plapply( 1:length( cychp.names ), extract_genotypes,
        cychp.files, cychp.names, output.temp, verbose)#, mc.cores=cychp.attime );

    return( genotypes );
}


extract_genotypes <- function( ii, cychp.files, output.temp, verbose ) {

    # pysplt <- system.file( "exec/splitter.py", package="affy2snpmatrix" );
    # pysplt <- paste0( "python ", pysplt, " -i ", cychp.files[ ii ], " -o ", output.temp, " -t cyhd" )

    # screen_cyhd_snp( verbose, paste0( "Parsing cyhd.cychp.txt file '", basename( cychp.files[ ii ] ) ,"'." ) );
    # system( pysplt );

    name <- gsub( ".cyhd.cychp.txt", "", basename( cychp.files[ ii ] ) );

    pypa <- system.file( "exec/parser.py", package="affy2sv" );
    pypa <- paste0( "python ", pypa, " -i ", cychp.files[ ii ], " -o ", output.temp, " -s -c" )

    screen_cyhd_snp( verbose, "Calling 'parser':\n", pypa )

    system( pypa )


    p <- 1;
    file.name <- unlist( strsplit( basename( cychp.files[ ii ] ), '\\.' ) )[ p ];
    while( file.name == "" ) {
        p = p + 1;
        file.name <- unlist( strsplit( basename( cychp.files[ ii ] ), '\\.' ) )[ p ];
    }


    genotypeset <- file.path( output.temp, paste0( name, ".cyhd.cychp.genotype.f.txt" ), fsep=.Platform$file.sep );
    # copynumberset <- file.path( output.temp, paste0( name, ".cyhd.cychp.cnds.f.txt" ), fsep=.Platform$file.sep );




    screen_cyhd_snp( verbose, paste0( "Loading genotypes for individual '", basename( cychp.files[ ii ] ), "'." ) );
    screen_cyhd_snp( verbose, paste0( "    '", genotypeset ,"'." ) );
    gds <- tryCatch( { read.delim( genotypeset , header=TRUE, as.is=TRUE, sep="\t" ); },
        warnning=function( w ) { screen_cyhd_snp( TRUE, paste0( "Invalid file provided in \"genotypeset\", '", genotypeset ,"'.\n" ), w ); stop( "HALT." ); },
        error=function( e ) { screen_cyhd_snp( TRUE, paste0( "Invalid file provided in \"genotypeset\", '", genotypeset ,"'.\n" ), e ); stop( "HALT." ); } )

    # screen_cyhd_snp( verbose, paste0( "Loading copy number info. for individual '", basename( cychp.files[ ii ] ), "'." ) );
    # screen_cyhd_snp( verbose, paste0( "     '", copynumberset ,"'." ) );
    # cnds <- tryCatch( { read.delim( copynumberset, header=TRUE, as.is=TRUE, sep="\t" ); },
    #     warnning=function( w ) { screen_cyhd_snp( TRUE, paste0( "Invalid file provided in \"copynumberset\" '", copynumberset ,"'\n." ), w ); stop( "HALT." ); },
    #     error=function( e ) { screen_cyhd_snp( TRUE, paste0( "Invalid file provided in \"copynumberset\" '", copynumberset ,"'\n." ), e ); stop( "HALT." ); } )

    # screen_cyhd_snp( verbose, "Extracting and filtering genotypes." );
    # relation <- gds$Index + 1;
    # names <- cnds[ relation, "ProbeSetName" ];

    # rownames( gds ) <- names;
    rownames( gds ) <- gds$ProbeSetName;
    # rownames( cnds ) <- cnds$ProbeSetName;

    # relation <- intersect( rownames( gds ), rownames( cnds ) );


    calls <- gds$Call;
    calls[ calls=="AA" ] <- 0;
    calls[ calls=="AB" ] <- 1;
    calls[ calls=="BB" ] <- 2;
    names( calls ) <- gds$ProbeSetName;


    c <- list();
    c$genotypes <- calls;
    c$names <- gds$ProbeSetName;

    return( c );
}




process_files_as_ped <- function( cychp.files, cychp.attime, allele.A, allele.B,
    strand, output.temp, verbose ) {

    screen_cyhd_snp( verbose, "Extracting calls from cychp." );
    calls.ind <- plapply( 1:length( cychp.files ), extract_genotypes,
        cychp.files, output.temp, verbose)#, mc.cores= cychp.attime );


    screen_cyhd_snp( verbose, "Flatting genotypes." );
    common <- Reduce( intersect, Map( function( x ) { x$names; }, calls.ind ) )
    calls.ind <- Map( function( x ){ x$genotype[ common ] }, calls.ind )

    calls <- calls.ind[[ 1 ]]
    if( length( calls.ind ) > 1 ) {
        for( ii in 2:length( calls.ind ) ) {
            calls <- cbind( calls, calls.ind[[ ii ]] );
        }
    }




    screen_cyhd_snp( verbose, "Getting A|B Alleles." );
    alleleA <- alleleB <- matrix( "", nrow=nrow( calls ), ncol=ncol( calls ), dimnames=dimnames( calls ) );

    allComplA <- c( "A", "C", "G", "T" )[ match( allele.A, c( "T", "G", "C", "A" ) ) ];
    forwA <- ifelse( strand=="+", allele.A, allComplA );

    allComplB <- c( "A", "C", "G", "T" )[ match( allele.B, c( "T", "G", "C", "A" ) ) ]
    forwB <- ifelse( strand == "+", allele.B, allComplB )


    screen_cyhd_snp( verbose, "Creating CALLS dataset." );
    for( ii in 1:ncol( calls ) ) {
      alleleA[ , ii ] <- ifelse( calls[ , ii ] < 3, forwA, forwB );
      alleleB[ , ii ] <- ifelse( calls[ , ii ] < 2, forwA, forwB );
    }

    screen_cyhd_snp( verbose, "Coercing TPED to matrix." );

    p1 <- seq( 1, ncol( calls )*2, 2 );
    p2 <- seq( 2, ncol( calls )*2, 2 );

    ped <- matrix( "", ncol=ncol( calls ) * 2, nrow=nrow( calls ) );
    for( ii in 1:ncol( calls ) ) {
        ped[ , p1[ ii ] ] <- alleleA[ , ii ];
        ped[ , p2[ ii ] ] <- alleleB[ , ii ];
    }

    rownames( ped ) <- common;

    return( ped );
}




