



# cel.files=gw6.celPath(); output.name="rawData"; output.type = "mad"
# cel.platform = "affy6"; cel.validate = FALSE; cel.genome = "hg19"
# cel.attime = 4; verbose = TRUE

gw2madfile <- function(cel.files, output.name, output.type = "mad",
    cel.platform = "affy6", cel.validate = TRUE, cel.genome = "hg19",
    cel.attime = 4, remove.tmp = TRUE, fill.24 = TRUE, verbose = FALSE) {

    # Getting input arguments
    cel.platform <- value(cel.platform, c("affy6", "affy5"))
    cel.genome <- value(cel.genome, c("hg18", "hg19"))
    output.type <- value(output.type, c("mad", "penncnv"))
    output.temp <- RandomName(prefix="tmp.gw.mad.")
    crlmm.lim <- 30;
    crlmm.oc <- c(2, 1e3)
    if (cel.files == "") { # TODO: Must be checked on windows
        cel.files <- "./"
    }
    cdfname <- ifelse(cel.platform=="affy6",
        "genomewidesnp6","genomewidesnp5");
    # /Getting input arguments


    # Checking input arguments
    if (is.na(cel.platform)) {
        screen_gw_mad(TRUE, TRUE, "Argument 'cel.platform' must be 'affy6'",
            " or 'affy5' for Genomewide 6.0 or 5.0, respectively.")
    }

    if (is.na(cel.genome)) {
        screen_gw_mad(TRUE, TRUE, "Argument 'cel.genome' must be 'hg18' ",
            "or hg19'.")
    }

    if(is.na(output.type)) {
        screen_gw_mad(TRUE, TRUE, "Argument 'output.type' must be 'mad' ",
            "or 'penncnv'.")
    }

    if(is.na(output.name) | output.name == '') {
        screen_gw_mad(TRUE, TRUE, "Argument 'output.name' must be filled.")
    }
    # /Checking input arguments


    # Showing the input arguments
    m <- paste0( "\n-------------------------\n",
        " - Input:        ", cel.files, "\n",
        " - Output type:  ", output.type, "\n",
        " - Output name:  ", output.name, "\n",
        " - Temp path:    ", output.temp, "\n",
        " - Platform:     ", cel.platform, "\n",
        " - Genome:       ", cel.genome, "\n",
        " - CEL x time:   ", cel.attime, "\n",
        " - CEL Validate: ", cel.validate );
    m <- paste0( m, "\n-------------------------" )
    screen_gw_mad(verbose, FALSE, m)
    # /Showing the input arguments

    # Locating CEL files
    screen_gw_mad(verbose, FALSE, "Locating CEL files.")
    cel.list <- list.celfiles(cel.files, full.names = TRUE)
    if (length(cel.list) == 0) {
        screen_gw_mad(TRUE, TRUE, "No CEL files in directory '", cel.files,
            "'." )
    }
    # /Locating CEL files

    # Validating CEL files
    if(cel.validate) {
        screen_gw_mad(verbose, FALSE, "Validating CEL files.");
        tryCatch( {
            validCEL( cel.list )
        }, warnning=function( w ){
            screen_gw_mad(TRUE, TRUE, "Bad (w) CEL files.\n", w )
        }, error=function( e ){
            screen_gw_mad(TRUE, TRUE, "Bad (e) CEL files.\n", e )
        });
    }
    # /Validating CEL files

    unlink(output.temp, recursive = TRUE, force = TRUE)
    unlink(output.name, recursive = TRUE, force = TRUE)
    dir.create(output.temp, showWarnings=FALSE)

    screen_gw_mad(verbose, FALSE, "Staring the CEL files analysis.")
    data <- GWProcess(cel.list, cel.genome, cel.attime, output.name,
        output.temp, output.type, cdfname, crlmm.oc, verbose)

    if(output.type == "mad") {
        screen_gw_mad(verbose, FALSE, "Creating output directory.")
        dir.create(output.name, showWarnings = FALSE)
        GenomeWideAsMad(data, output.name, fill.24, verbose)
    }
    else {
       GenomeWideAsPennCNV(data, output.name, verbose)
    }

    if (remove.tmp) {
        unlink(output.temp, recursive=TRUE, force=TRUE)
    }
}


# normalize_baf
# ------------------------------------------------------------------------------
# 12/07/2013
# Function to re-normalize the BAF not using the centers of the genotype-groups
# but using statistic centers ( 0.05, 0.50, 0.95 ).

normalize_baf <- function( baf, gtype ) {
    # gtype <- ds[ , "GType" ]
    # baf <- as.numeric( as.vector( ds[ , "B.Allele.Freq" ] ) )

    frS <- rep( NA, length( gtype ) )
    result <- rep( NA, length( gtype ) )

    result[ gtype == "AA" ] <- baf[ gtype == "AA" ]
    result[ gtype == "AB" ] <- baf[ gtype == "AB" ]
    result[ gtype == "BB" ] <- baf[ gtype == "BB" ]

    # Apply normalization to the AA and BB
    thAA <- 0.05;
    thAB <- 0.5;
    thBB <- 0.95;
    result[ result < thAA ] <- 0;
    result[ result > thBB ] <- 1;
    result[ thAA <= result & result < thAB & gtype != "AB" & !is.na( result ) ] <-
        ( 0.5 * ( result[ thAA <= result & result < thAB & gtype != "AB" & !is.na( result ) ] - thAA ) ) / ( thAB - thAA );
    result[ thAB <= result & result < thBB & gtype != "AB" & !is.na( result ) ] <-
        0.5 + ( ( 0.5 * ( result[ thAB <= result & result < thBB & gtype != "AB"& !is.na( result ) ] - thAB ) ) / ( thBB - thAB ) );

    return( result );
}
