
# screen_gw_snp
# ------------------------------------------------------------------------------
# 17/07/2013

screen_gw_snp <- function( verbose, ... ) {
    if( verbose ) {
        message( paste0( "[ gw2snpmatrix ]: ", ... ) );
    }
}

# gw2snpmatrix
# ------------------------------------------------------------------------------
# 18/07/2013
# Function to create an object which contains a SnpMatrix and a mapping dataset
# throw a cohort of CEL files from Affymetrix Genome-Wide SNP 5.0 or
# Affymetrix Genome-Wide SNP 6.0.

gw2snpmatrix <- function ( cel.files, cel.platform = "affy6", cel.attime = 4, 
    cel.validate = TRUE, output.type = "snpmatrix", output.name = NA, verbose = FALSE ) {

    # Getting input arguments
    cel.platform <- value( cel.platform, c( "affy6", "affy5" ) )
    if ( cel.files == "" ) { # TODO: Must be checked on windows
        cel.files <- "./"; 
    }
    output.type <- value( output.type, c( "snpmatrix", "plink" ) )
    output.temp <- "tmp.gw.snpmatrix"
    # /Getting input arguments


    # Checking input arguments
    if ( is.na( cel.platform ) ) {
        screen_gw_snp( TRUE, "Argument 'cel.platform' must be 'affy6' or 'affy5' for Genomewide 6.0 or 5.0, respectively." )
        stop( "HALT." )
    }
    if( output.type == "" ) {
        screen_gw_snp( TRUE, "Argument 'output.type' must be 'snpmatrix' or 'plink'." )
        stop( "HALT." )
    }
    if( output.type == "plink" & is.na( output.name ) ) {
        screen_gw_snp( TRUE, "If 'output.type' is 'plink', 'output.name' must be filled." )
        stop( "HALT." )   
    }
    # /Checking input arguments


    # Showing the input arguments
    m <- paste0( "\n-------------------------\n",
        " - Input:      ", cel.files, "\n",
        " - Platform:   ", cel.platform, "\n",
        " - CEL x time: ", cel.attime, "\n",
        " - Output:     ", output.type );
    if( output.type == "plink" ) {
        m <- paste0( m, "\n - Output f.:   ", output.name )
    }
    m <- paste0( m, "\n-------------------------" )
    screen_gw_snp( verbose, m )
    # /Showing the input arguments


    # Locating CEL files
    screen_gw_snp( verbose, "Locating CEL files." );
    cel.list <- list.celfiles( cel.files, full.names=TRUE );
    if ( length( cel.list ) == 0 ) { 
        screen_gw_snp( verbose, paste0( "No CEL files in directory '", cel.files, "'." ) );
        stop( "HALT." );
    }
    # /Locating CEL files


    # Validating CEL files
    if( cel.validate ) {
        screen_gw_snp( verbose, "Validating CEL files." );
        tryCatch( { validCEL( cel.list ) }, 
            warnning=function( w ){ screen_gw_snp( TRUE, paste0( "Bad (w) CEL files.\n", w ) ); stop( "HALT." ); },
            error=function( e ){ screen_gw_snp( TRUE, paste0( "Bad (e) CEL files.\n", e ) ); stop( "HALT." ) } );
    }
    # /Validating CEL files


    # Creating the MAP content
    screen_gw_snp( verbose, "Adding annotation to main container." );
    if( cel.platform=="affy6" ) {
        con <- pd.genomewidesnp.6@getdb();
    } else {
        con <- pd.genomewidesnp.5@getdb();
    }
    snp.t <-  dbGetQuery( con, "select * from featureSet" );
    # /Creating the MAP content


    if( output.type == "snpmatrix" ) {
        # if no output.type is specified we create an object containing the
        # map content and the SnpMatrix to return it.

        screen_gw_snp( verbose, "'output.type' specified as 'snpmatrix', ",
            "so a container for MAP and SnpMatrix will be created ",
            "and returned." );


        screen_gw_snp( verbose, "Generating MAP content." );

        map <- snp.t[ , c( "dbsnp_rs_id", "chrom", "physical_pos", "allele_a", "allele_b", "strand" ) ];
        rownames( map ) <- snp.t[ , "man_fsetid" ];
        names( map )[ 2:3 ] <- c( "chromosome", "position" );

        object = list()
        object$genotype <- create_snp_matrix( cel.list, verbose );
        object$map <- map;
        object$map <- object$map[ colnames( object$genotype ), ];
        return( object );
    } 
    else {
        # if output.type is specified it contains the 'name' we will use to 
        # create the map and the ped files (plink output).
        screen_gw_snp( verbose, "'output.type' specified so a TPED file ",
            "will be created and stored to disk." );

        screen_gw_snp( verbose, "Selecting SNP data from annotation file." );
        output.auxdata <- cbind( 
            snp.t[ , c( "chrom", "dbsnp_rs_id" ) ],
            rep( 0, dim( snp.t )[ 1 ] ),
            snp.t[ , c( "physical_pos" ) ] );
        rownames( output.auxdata ) <- snp.t[ , "man_fsetid" ];

        screen_gw_snp( verbose, "Creating the TPED content," );
        name <- paste0( output.name, ".tped" );
        create_ped_file( cel.list, cel.files, cel.attime, 
            snp.t$allele_a, snp.t$allele_b, snp.t$strand, output.auxdata, name, verbose );
    }

}


# # create_single_snp_matrix
# # ------------------------------------------------------------------------------
# # 12/07/2013
# # Function to process the whole cohorts creating the container for the SnpMatrix
# # and the MAP file.

# create_snp_matrix <- function( cel.list, cel.attime, verbose ) {

#     # Perform the analysis
#     # if( length( cel.index ) > 1 ) {
#     #     screen_gw_snp( verbose, "Starting multi-cohort analysis." );
#     # }
    
#     object <- list()
#     object$genotype <- create_single_snp_matrix( cel.list, verbose );

#     # Creating 'object'
#     # screen_gw_snp( verbose, "Creating main container." );
        
#     # # Filling the SnpMatrix
#     # screen_gw_snp( verbose, "Joining SnpMatrix to main container." );

#     # for ( ii in 1:length( result ) ) {
#     #     if ( is.null( object$genotype ) ) {
#     #         object$genotype <- result[[ ii ]]
#     #     } else {
#     #         if ( class( result[[ ii ]] ) == "SnpMatrix" ) {
#     #             object$genotype <- rbind( object$genotype, result[[ ii ]] );
#     #         }
#     #     }
#     # }

#     return( object );
# }


# create_single_snp_matrix
# ------------------------------------------------------------------------------
# 12/07/2013
# Function to process a single set with crlmm and create a SnpMatrix.

create_snp_matrix <- function( cel.files, verbose ) {
    
    # Getting the CEL to be processed
    # cel.files <- cel.files[ cel.index[ 1 ]:cel.index[ 2 ] ];
    # /Getting the CEL to be processed

    # screen_gw_snp( TRUE, "cel.files: ", cel.files );
    screen_gw_snp( verbose, "Creating SnpSet." );
    tryCatch( { crlmm.out <- crlmm::crlmm( cel.files ); }, 
        warnning=function( w ){ screen_gw_snp( TRUE, paste0( "Warning performing 'CRLMM'.\n", w ) ); stop( "HALT." ); },
        error=function( e ){ screen_gw_snp( TRUE, paste0( "Error performing 'CRLMM'.\n", e ) ); stop( "HALT." ); } );

    screen_gw_snp( verbose, "Coercing calls to SnpMatrix." );
    calls.ind <- tryCatch( { new( "SnpMatrix", t( calls( crlmm.out ) ) - 1L ); },
        warning=function( w ){ screen_gw_snp( TRUE, paste0( "Warning coercing to SnpMatrix.\n", w ) ); stop( "HALT." ); },
        error=function( e ){ screen_gw_snp( TRUE, paste0( "Error coercing to SnpMatrix.\n", e ) ); stop( "HALT." ); } );

    return( calls.ind );
}


# create_ped_file
# ------------------------------------------------------------------------------
# 18/07/2013

create_ped_file <- function( cel.list, cel.files, cel.attime, 
    allele.A, allele.B, strand, output.auxdata, output.name, verbose ) {

    # # Checking for the value of 'output.auxdata'
    # if( is.na( output.auxdata ) ) {
    #     screen_gw_snp( verbose, "Creating dummy values for 'output.auxdata'." );
    #     sample_id <-  sapply( list.celfiles( cel.files, full.names=FALSE ), function( x ) { return( gsub( ".CEL", "", x ) ); } )
    #     ns <- length( sample_id )
    #     # 1: Family Id; 2: Sample Id; 3: Paternal Id; 4: Maternal Id; 5: Gender; 6: Affection
    #     output.auxdata <- data.frame( rep( "FAM1", ns ), sample_id, rep( "PTR1", ns ), rep( "MTR1", ns ), rep( 0, ns ), rep( 0, ns ) );
    #     colnames( output.auxdata ) <- c( "Family Id", "Sample Id", "Paternal Id", "Maternal Id", "Gender", "Affection" );
    # }
    # else {
    #     if( ncol( output.auxdata ) != 6 ) {
    #         screen_gw_snp( TRUE, "Number of columns in 'output.auxdata' has to be 6:\n    1: Family Id; 2: Sample Id; 3: Paternal Id; 4: Maternal Id; 5: Gender; 6: Affection" );
    #         stop( "HALT." );
    #     }
    #     if( nrow( output.auxdata ) != length( sample_id  ) ) {
    #         screen_gw_snp( TRUE, "Number of rows in 'output.auxdata' does not match in number with the population in 'cel.files'." );
    #     }
    #     screen_gw_snp( TRUE, "Correct dimensions for 'output.auxdata'." );
    # }
    # # /Checking for the value of 'output.auxdata'


    ped <- create_single_ped_file( cel.list, allele.A, allele.B, 
        strand, verbose );

    #screen_gw_snp( verbose, "Flatting the batch results." );
    #ped <- do.call( "rbind", result );

    screen_gw_snp( verbose, "Binding common data." );
    ped <- cbind( output.auxdata[ rownames( ped ), ], ped );

    # screen_gw_snp( verbose, "Coercing PED to matrix before writing." );
    # ped <- as( ped, "matrix" );
    
    screen_gw_snp( verbose, "Writing TPED file to disk '" , output.name, "'." );
    write.table( ped, file=output.name, quote=FALSE, row.names=FALSE, col.names=FALSE );

    return( NA );
}


# create_single_ped_file
# ------------------------------------------------------------------------------
# 18/07/2013

create_single_ped_file <- function( cel.files, allele.A, allele.B, 
    strand, verbose ) {

    # Getting the CEL to be processed
    # cel.files <- cel.list[ cel.index[ 1 ]:cel.index[ 2 ] ];
    # /Getting the CEL to be processed


    #screen_gw_snp( verbose, paste0( "Creating SnpSet (" , cel.index[ 1 ], " - ", cel.index[ 2 ], ")." ) );
    tryCatch( { crlmm.out <- crlmm::crlmm( cel.files ); }, 
        warnning=function( w ){ screen_gw_snp( TRUE, paste0( "Warning performing 'CRLMM'.\n", w ) ); stop( "HALT." ); },
        error=function( e ){ screen_gw_snp( TRUE, paste0( "Error performing 'CRLMM'.\n", e ) ); stop( "HALT." ); } );
    
    screen_gw_snp( verbose, "Getting calls from cnSet." );
    calls <- calls( crlmm.out );
    
    alleleA <- alleleB <- matrix( "", nrow=nrow( calls ), ncol=ncol( calls ), dimnames=dimnames( calls ) );
    
    allA <- allele.A;
    allComplA <- c( "A","C","G","T" )[ match( allA, c( "T", "G", "C", "A" ) ) ];
    forwA <- ifelse( strand=="+", allA, allComplA );
    
    allB <- allele.B;
    allComplB <- c( "A","C","G","T" )[ match( allB, c( "T", "G", "C", "A" ) ) ]
    forwB <- ifelse( strand == "+", allB, allComplB )

    for( ii in 1:ncol( calls ) ) {
      alleleA[ ,ii ] <- ifelse( calls[ ,ii ] < 3, forwA, forwB );
      alleleB[ ,ii ] <- ifelse( calls[ ,ii ] < 2, forwA, forwB );
    }
    
    screen_gw_snp( verbose, "Coercing PED to matrix." );
    
    p1 <- seq( 1, ncol( calls )*2, 2 );
    p2 <- seq( 2, ncol( calls )*2, 2 );

    ped <- matrix( "", ncol=ncol( calls ) * 2, nrow=nrow( calls ) );
    for( ii in 1:ncol( calls ) ) {
        ped[ , p1[ ii ] ] <- alleleA[ , ii ];
        ped[ , p2[ ii ] ] <- alleleB[ , ii ];
    }

    rownames( ped ) <- rownames( calls );


    # screen_gw_snp( verbose, "Creating single container." );
    # ped <- matrix( "", ncol=nrow( alleleA ) * 2, nrow=ncol( alleleA ) )
    # for( ii in 1:ncol( calls ) ) {
    #     ped[ ii, ( 1:nrow( alleleA ) ) * 2 - 1 ] <- alleleA[ ,ii ];
    #     ped[ ii, ( 1:nrow( alleleB ) ) * 2 ] <- alleleB[ ,ii ];
    # }
    # rownames( ped ) = sapply( colnames( calls ), function( x ) { return( gsub( ".CEL", "", x ) ) } );
    
    return( ped );
}